#!/usr/bin/perl -w
###############################################################
# Simple Shopping Basket                      Version: v1.01  #
###############################################################
# Written By:   Mark R Hives                                  #
# Filename:     m_shop_basket.pl                              #
# Release Date:  8-Apr-2004                                   #
#               12-Jan-2003  [v1.00]                          #
#                                                             #
# Description:                                                #
#  The objective of this project is to create a reasonably    #
#  secure and robust shopping system that can be incorporated #
#  in to any existing website with the minimum of fuss.       #
#                                                             #
# How to use this script:                                     #
#  I haven't had the time to prepare a lengthy discussion so  #
#  here is the bare bones version. Make a form and include    #
#  all the elements listed below as either hidden fields, or  #
#  in the case of the Qty a text field, the forms method      #
#  property should be set to post. Also please note that the  #
#  price must be of the form "x.xx" and that no fields should #
#  contain the pipe symbol.                                   #
#                                                             #
#  pc     |The Products unique Code number |Alphanumeric      #
#  pn     |A descriptive Product Name      |Alphanumeric      #
#  pq     |Product Qty                     |Numeric (Integer) #
#  pp     |The Products Price              |Decimal Number    #
#  action |Required but always hidden      |set to "do_upd"   #
#                                                             #
#  To make a "show cart" button you should omit the ALL of    #
#  fields from the form or make a simple hyperlink to the     #
#  script.                                                    #
###############################################################

my (%fd, @pc, @pn, @pq, @pp, $buf, $idx);

#This script always outputs html so output type header now.
print "content-type: text/html\n";

@pc = ();
@pn = ();
@pq = ();
@pp = ();
read_cookie();

%fd = ();
read_formdata();

if ($fd{'action'} eq 'do_upd' ) {
   die_msg("Error: do_upd - Missing p?  ") unless (($fd{'pc'}) && ($fd{'pn'}) && ($fd{'pq'}) && ($fd{'pp'}));
   die_msg("Error: do_upd - Qty  <  1   ") unless (int($fd{'pq'}) > 0);
   die_msg("Error: do_upd - Price Format") unless ($fd{'pp'} =~ m/^[0-9]+\.[0-9]{2}$/i);

   $idx = find_item($fd{'pc'});
   if ($idx == -1) {
      push @pc, $fd{'pc'};
      push @pn, $fd{'pn'};
      push @pq, $fd{'pq'};
      push @pp, $fd{'pp'};

   } else {
      $pc[$idx] = $fd{'pc'};
      $pn[$idx] = $fd{'pn'};
      $pq[$idx] = $fd{'pq'};
      $pp[$idx] = $fd{'pp'};
   }

} elsif ($fd{'action'} eq 'do_rmv' ) {
   die_msg("Error: do_rmv - arg1") unless ($fd{'pc'});
   $idx = find_item($fd{'pc'});
   die_msg("Error: do_rmv - arg2") if ($idx == -1);

   if ($idx != (scalar(@pc) - 1)) {
      $pc[$idx] = $pc[-1];
      $pn[$idx] = $pn[-1];
      $pq[$idx] = $pq[-1];
      $pp[$idx] = $pp[-1];
   }
   pop @pc;
   pop @pn;
   pop @pq;
   pop @pp;

} elsif ($fd{'action'} eq 'basket_action') {
  foreach $buf (%fd) {
     if ($buf =~ m/^RM_(.+)$/i) {
        output_rmform($1);
        exit;
     } elsif ($buf =~ m/^CH_(.+)$/i) {
        output_upform($1);
        exit;
     }
  }

}

output_cookie();
output_header();
show_basket();
output_footer();

############################################################################

sub format_time {
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)
   = (gmtime($_[0] || time()));
  return sprintf("%s, %d-%s-%d %02d:%02d:%02d GMT",
         (qw(Sun Mon Tue Wed Thu Fri Sat))[$wday], $mday,
         (qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec))[$mon],
         ($year + 1900) ,
         $hour,$min,$sec);
}

sub show_basket {
   my($x, $t1);
   $t1 = 0;
   print "<form method=post action=\"m_shop_basket.pl\">\n",
         "<input type=hidden name=action value=basket_action>\n",
         "<table border=1 width=600>\n",
         "<tr><th width=\"15%\">CODE</th>\n",
         "    <th width=\"45%\">NAME</th>\n",
         "    <th width=\"10%\">QTY</th>\n",
         "    <th width=\"10%\">PRICE</th>\n",
         "    <th width=\"20%\">&nbsp;</th>\n</tr>\n";

   for ($x=0; $x < scalar(@pc); $x++) {
      print "<tr><td>", $pc[$x], "</td>\n",
            "    <td>", $pn[$x], "</td>\n",
            "    <td>", $pq[$x], "</td>\n",
            "    <td>", $pp[$x], "</td>\n";
      print "    <td>",
            "<input type=submit name=CH_",$pc[$x]," value=\"Change\">",
            "<input type=submit name=RM_",$pc[$x]," value=\"Remove\">",
            "</td>\n</tr>\n";
      $t1 = $t1 + ($pq[$x] * $pp[$x]);
   }
   if (scalar(@pc) == 0) {
      print ("<tr><td colspan=5>YOUR BASKET IS EMPTY</td></tr>");
   } else {
      print ("<tr><td colspan=2 align=right>Sub-Total: </td>\n",
             "<td>&nbsp;</td>\n");
      printf("<td>%.2f</td>\n", $t1);
      print ("<td>&nbsp;</td></tr>");
   }
   print "</table>\n</form>\n";
}


sub output_chform1 {
   my($chtype);
   $chtype = $_[1];
   die_msg("R/L Error in output_chform1() !") if (($chtype != 1) && ($chtype != 2));

   $idx = find_item($_[0]);
   die_msg("Item Not Found In Your Basket !!!") if ($idx == -1);

   output_header();
   print "<form method=post action=\"m_shop_basket.pl\">\n",
         "<table width=200 border=1>\n",
         "<tr><td colspan=2>", $pn[$idx], "</td>\n",
         "<tr><td>Code:</td>\n",
         "    <td>", $pc[$idx], "</td>\n</tr>\n",
         "<tr><td>Qty:</td>\n",
         "    <td>",
          "<input type=hidden name=pc value=\"", $pc[$idx], "\">\n";
   if ($chtype == 1) {
      print "<input type=hidden name=pn value=\"", $pn[$idx], "\">\n",
            "<input type=text   name=pq value=\"", $pq[$idx], "\" size=4>\n",
            "<input type=hidden name=pp value=\"", $pp[$idx], "\">\n";
   } elsif ($chtype == 2) {
      print $pq[$idx];
   }
   print "</td>\n</tr>\n",
         "<tr><td>&nbsp;</td>\n<td>";

   if ($chtype == 1) {
      print "<input type=hidden name=action value=\"do_upd\">\n",
            "<input type=submit name=submit value=\"update\">\n",
   } elsif ($chtype == 2) {
      print "<input type=hidden name=action value=\"do_rmv\">\n",
            "<input type=submit name=submit value=\"remove\">\n",
   }
   print "</td>\n",
         "\n";
   print "</table>\n</form>\n";
   output_footer();
}

sub output_upform {
  output_chform1($_[0], 1);
}
sub output_rmform {
  output_chform1($_[0], 2);
}

sub die_msg    {
   output_header();
   print @_;
   #print %ENV;
   output_footer();
   exit;
}

sub read_formdata {
   my (@pairs, $pair, $name, $value, $buffer);
   if ($ENV{'REQUEST_METHOD'} eq 'POST') {
      read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
      @pairs = split /&/, $buffer;
   } else {
      @pairs = split /&/, $ENV{'QUERY_STRING'};
   }

   foreach $pair (@pairs) {
      ($name, $value) = split /=/, $pair;
      $name =~ tr/+/ /;
      $name =~ s/%([\da-f]{2})/chr(hex($1))/egi;
      $value =~ tr/+/ /;
      $value =~ s/%([\da-f]{2})/chr(hex($1))/egi;

      $value =~ tr/\|\\\/;/____/;
      $fd{$name} = $value;
   }
}

sub read_cookie   {
   my (@pairs, $pair, $name, $value, @buffer, $x);
   @pairs = split /; /, $ENV{'HTTP_COOKIE'};
   foreach $pair (@pairs) {
   ($name, $value) = split /=/, $pair;
      if ($name eq 'msb') {
         $value =~ tr/+/ /;
         $value =~ s/%([\da-f]{2})/chr(hex($1))/egi;
         @buffer = split /\|/, $value;
         last;
      }
   }
   if (@buffer) {
      for ($x=0; $x < (scalar(@buffer) - (scalar(@buffer) % 4)); $x+=4) {
         push @pc, $buffer[$x];
         push @pn, $buffer[($x+1)];
         push @pq, $buffer[($x+2)];
         push @pp, $buffer[($x+3)];

         $pq[-1] = 1      if (int($pq[-1]) < 1);
         $pp[-1] = "0.00" if ($pp[-1] !~ m/^[0-9]+\.[0-9]{2}$/i);
      }
   }
}

sub output_cookie {
   my ($x, $msb);
   $msb = "";
   for ($x=0; $x < scalar(@pc); $x++) {
      $msb = $msb . $pc[$x] . '|' .
                    $pn[$x] . '|' .
                    $pq[$x] . '|' .
                    $pp[$x] . '|';
   }
   $msb =~ s/([^0-9a-z_\.!~*()-|])/sprintf("%%%02X", ord($1))/egi;

   print "Set-Cookie: msb=", $msb,
                   "; expires=", format_time(time() + (86400 * 30)), "\n";
}

sub find_item {
   my ($x);
   for ($x=0; $x < scalar(@pc); $x++) {
      return($x) if ($pc[$x] eq $_[0]);
   }
   return -1;
}

sub output_header {
   print "\n<html>\n<body>\n";
   print "<table width=100% border=0><tr><td>\n",
         "<font size=5 color=#333333>Shopping Basket</font>",
         "</td></tr></table>\n";
}

sub output_footer {
   print "<table width=100% border=0 cellpadding=0 cellspacing=0>\n",
         "<tr><td align=left width=50%>\n",
         "<font size=1 color=#999999>Simple Shopping Basket v1.01</font>\n",
         "</td><td align=right width=50%>",
         "<font size=1 color=#999999>&copy; 2003 Mark R Hives</font>\n",
         "</td></tr></table>\n";

   #print "\n<code><br>\n", $ENV{'HTTP_COOKIE'}, "\n</code>\n"; ######################
   print "</body>\n</html>";
}


